searchData={"items":[{"type":"module","title":"ucl","doc":null,"ref":"ucl.html"},{"type":"function","title":"ucl.to_json/1","doc":null,"ref":"ucl.html#to_json/1"},{"type":"extras","title":"README","doc":"","ref":"readme.html"},{"type":"extras","title":"UCL: BEAM-friendly bindings for `libUCL` - README","doc":"[LibUCL] is a human-friendly, automation-oriented parser and generator\nfor `Universal Configuration Language` or `UCL`. UCL's main benefits\nare:\n\n- human readable and editable with comments\n- machine parsable\n- conversion to/from YAML, JSON\n- macro and include-file support\n- lightning-fast parser\n\n`UCL` is a NIF-based binding to the reference implementation [LibUCL],\nand currently requires that library to be pre-installed on your system.\n\nIt does not use dirty schedulers, nor does it time-slice NIF calls, as\nthis would make the code significantly more complex for a function that\nis unlikely to be run frequently within the BEAM.\n\nUCL comes with friendly functions for *both* Elixir and Erlang, we are\nall friends here.","ref":"readme.html#ucl-beam-friendly-bindings-for-libucl"},{"type":"extras","title":"Installation & Compilation - README","doc":"","ref":"readme.html#installation-compilation"},{"type":"extras","title":"Dependencies - README","doc":"`libucl` is obviously a dependency, and the usual UNIX build chain is\nrequired.\n\nYou will need Erlang/OTP 25 or higher, because the build tool `enc`\nis a pre-compiled escript.\n\n- FreeBSD:  `textproc/libucl devel/rebar3`\n- OSX:      `brew install libucl`\n- Debian:   `build-essential` from base\n            `erlang-dev erlang-nox` from erlang-solutions\n            `rebar3` from https://rebar3.org/\n- OpenBSD:  `erlang-25 elixir` from base\n            `rebar3` from https://rebar3.org/\n\nOn some systems, LibUCL is missing and you'll need to build it.\nOn Debian, there is a conflicting compression library of the same name.","ref":"readme.html#dependencies"},{"type":"extras","title":"Building LibUCL from source - README","doc":"","ref":"readme.html#building-libucl-from-source"},{"type":"extras","title":"OpenBSD - README","doc":"```\n$ doas pkg_add pkgconf autogen autoconf-archive autoconf-2.69 \\\n    automake-1.11 libtool\n$ tar xzf libucl-*.gz\n$ cd libucl*\n$ ./autogen.sh\n$ ./configure\n$ make\n$ doas make install\n```","ref":"readme.html#openbsd"},{"type":"extras","title":"Debian - README","doc":"```\n$ sudo apt install autoconf libcurl4-openssl-dev automake libtool \\\n    autoconf-archive pkg-config\n$ tar xzf libucl-*.gz\n$ cd libucl*\n$ ./autogen.sh\n$ ./configure --prefix=/usr\n$ make\n$ sudo make install\n```","ref":"readme.html#debian"},{"type":"extras","title":"Installation - README","doc":"Via git clone:\n\n```\n$ export PATH=$PATH:/usr/local/lib/erlang25/bin:$HOME/.mix\n$ mix local.rebar\n$ rebar3 do clean, compile, ct\n```\n\nVia [hex] in the usual fashion:\n\n- Elixir: add `{:ucl, \"~> 1.0\"}` to `mix.exs`\n- Erlang: add `{ucl, \"1.1.0\"}` to `rebar.config`\n\nCompilation should work on all UNIX-like OS out of the box. There is\no Windows support planned, but if you can compile `libucl` on Windows,\nget in touch.\n\nTests are available as usual via `rebar3 ct`, and long-running property\ntests via `rebar3 proper`. The property tests are effectively fuzzing\nthe NIF by injecting random `binary()` data and expecting it to return\na classic `{ok | error, any()}` tuple, and take a considerable amount of\ntime (several hours):\n\n```\n$ time rebar3 proper\n... 3 hours later ...\n00% {60000,66000}\n10% {54000,60000}\n09% {48000,54000}\n10% {42000,48000}\n10% {36000,42000}\n09% {30000,36000}\n09% {24000,30000}\n09% {18000,24000}\n09% {12000,18000}\n10% {6000,12000}\n9% {0,6000}\n===> 1/1 properties passed\nOK: Passed 500000 test(s).\n11005.99 real     11039.08 user        86.36 sys\n```\n\nAlthough `UCL` is written in Erlang, and uses `rebar3` ,it should\ncompile cleanly as a dependency on any BEAM language.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - README","doc":"```elixir\niex> UCL.to_json(\":foo\")\n     {:error, :ucl_invalid}\niex> \"foo: true\" |> UCL.to_json! |> Jason.decode!\n     %{\"foo\" => true}\n```\n\n```erlang\n1> ucl:to_json(<<\"foo: true\">>).\n    {ok,<<\"{\\n    \\\"foo\\\": true\\n}\">>}\n```\n\nThe implementation is intended for functional transformation of config\ndata held in memory, and thus a large portion of the LibUCL API has been\nsummarily ignored. If you need something please let us know.","ref":"readme.html#usage"},{"type":"extras","title":"Universal Configuration Language Syntax - README","doc":"UCL is heavily infused by `nginx` configuration as the example of a\nconvenient configuration system. However, UCL is fully compatible with\n`JSON` format and is able to parse json files. For example, you can\nwrite the same configuration in the following ways:\n\n* in nginx like:\n\n```nginx\nparam = value;\nsection {\n    param = value;\n    param1 = value1;\n    flag = true;\n    number = 10k;\n    time = 0.2s;\n    string = \"something\";\n    subsection {\n        host = {\n            host = \"hostname\";\n            port = 900;\n        }\n        host = {\n            host = \"hostname\";\n            port = 901;\n        }\n    }\n}\n```\n\n* or in JSON:\n\n```json\n{\n    \"param\": \"value\",\n    \"param1\": \"value1\",\n    \"flag\": true,\n    \"subsection\": {\n        \"host\": [\n        {\n            \"host\": \"hostname\",\n            \"port\": 900\n        },\n        {\n            \"host\": \"hostname\",\n            \"port\": 901\n        }\n        ]\n    }\n}\n```","ref":"readme.html#universal-configuration-language-syntax"},{"type":"extras","title":"Improvements to the json notation. - README","doc":"There are various things that make ucl configuration more convenient for\nediting than strict json:","ref":"readme.html#improvements-to-the-json-notation"},{"type":"extras","title":"General syntax sugar - README","doc":"* Braces are not necessary to enclose a top object: it is automatically\n  treated as an object:\n\n```json\n\"key\": \"value\"\n```\nis equal to:\n```json\n{\"key\": \"value\"}\n```\n\n* There is no requirement of quotes for strings and keys, moreover, `:`\n  may be replaced `=` or even be skipped for objects:\n\n```nginx\nkey = value;\nsection {\n    key = value;\n}\n```\nis equal to:\n```json\n{\n    \"key\": \"value\",\n    \"section\": {\n        \"key\": \"value\"\n    }\n}\n```\n\n* No commas mess: you can safely place a comma or semicolon for the last\n  element in an array or an object:\n\n```json\n{\n    \"key1\": \"value\",\n    \"key2\": \"value\",\n}\n```","ref":"readme.html#general-syntax-sugar"},{"type":"extras","title":"Automatic arrays creation - README","doc":"* Non-unique keys in an object are allowed and are automatically\n  converted to the arrays internally:\n\n```json\n{\n    \"key\": \"value1\",\n    \"key\": \"value2\"\n}\n```\nis converted to:\n```json\n{\n    \"key\": [\"value1\", \"value2\"]\n}\n```","ref":"readme.html#automatic-arrays-creation"},{"type":"extras","title":"Named keys hierarchy - README","doc":"UCL accepts named keys and organize them into objects hierarchy\ninternally. Here is an example of this process:\n\n```nginx\nsection \"blah\" {\n\tkey = value;\n}\nsection foo {\n\tkey = value;\n}\n```\n\nis converted to the following object:\n\n```nginx\nsection {\n\tblah {\n\t\tkey = value;\n\t}\n\tfoo {\n\t\tkey = value;\n\t}\n}\n```\n\nPlain definitions may be more complex and contain more than a single\nlevel of nested objects:\n\n```nginx\nsection \"blah\" \"foo\" {\n\tkey = value;\n}\n```\n\nis presented as:\n\n```nginx\nsection {\n\tblah {\n\t\tfoo {\n\t\t\tkey = value;\n\t\t}\n\t}\n}\n```","ref":"readme.html#named-keys-hierarchy"},{"type":"extras","title":"Convenient numbers and booleans - README","doc":"* Numbers can have suffixes to specify standard multipliers:\n    + `[kKmMgG]` - standard 10 base multipliers (so `1k` is translated to 1000)\n    + `[kKmMgG]b` - 2 power multipliers (so `1kb` is translated to 1024)\n    + `[s|min|d|w|y]` - time multipliers, all time values are translated to float number of seconds, for example `10min` is translated to 600.0 and `10ms` is translated to 0.01\n* Hexadecimal integers can be used by `0x` prefix, for example `key = 0xff`. However, floating point values can use decimal base only.\n* Booleans can be specified as `true` or `yes` or `on` and `false` or `no` or `off`.\n* It is still possible to treat numbers and booleans as strings by enclosing them in double quotes.","ref":"readme.html#convenient-numbers-and-booleans"},{"type":"extras","title":"General improvements - README","doc":"","ref":"readme.html#general-improvements"},{"type":"extras","title":"Comments - README","doc":"UCL supports different style of comments:\n\n* single line: `#`\n* multiline: `/* ... */`\n\nMultiline comments may be nested:\n```c\n# Sample single line comment\n/*\n some comment\n /* nested comment */\n end of comment\n*/\n```","ref":"readme.html#comments"},{"type":"extras","title":"Thanks - README","doc":"- Paul Davis, whose well documented [NIF examples] made writing this almost cake\n\n[NIF examples]: https://github.com/davisp/nif-examples\n[hex]: https://hex.pm/packages/ucl\n[LibUCL]: https://github.com/vstakhov/libucl","ref":"readme.html#thanks"},{"type":"extras","title":"LICENSE","doc":"BSD 2-Clause License\n\nCopyright (c) 2019-2038, SkunkWerks, GmbH\nAll rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, are permitted provided that the following conditions are met:\n\n1. Redistributions of source code must retain the above copyright notice, this\n   list of conditions and the following disclaimer.\n\n2. Redistributions in binary form must reproduce the above copyright notice,\n   this list of conditions and the following disclaimer in the documentation\n   and/or other materials provided with the distribution.\n\nTHIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"\nAND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\nIMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE\nDISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE\nFOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\nDAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR\nSERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER\nCAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,\nOR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE\nOF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.","ref":"license.html"}],"content_type":"text/plain","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}